package com.sun.org.apache.xpath.internal.jaxp;

import org.w3c.dom.traversal.NodeIterator;
import jdk.xml.internal.JdkXmlUtils;
import org.xml.sax.InputSource;
import javax.xml.xpath.XPathConstants;
import com.sun.org.apache.xpath.internal.res.XPATHMessages;
import javax.xml.xpath.XPathFunctionException;
import javax.xml.xpath.XPathExpressionException;
import org.w3c.dom.Node;
import com.sun.org.apache.xpath.internal.XPathContext;
import javax.xml.transform.TransformerException;
import com.sun.org.apache.xpath.internal.objects.XObject;
import javax.xml.namespace.QName;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import jdk.xml.internal.JdkXmlFeatures;
import com.sun.org.apache.xpath.internal.XPath;
import javax.xml.xpath.XPathVariableResolver;
import javax.xml.xpath.XPathFunctionResolver;
import javax.xml.xpath.XPathExpression;

public class XPathExpressionImpl implements XPathExpression
{
    private XPathFunctionResolver functionResolver;
    private XPathVariableResolver variableResolver;
    private JAXPPrefixResolver prefixResolver;
    private XPath xpath;
    private boolean featureSecureProcessing;
    boolean overrideDefaultParser;
    private final JdkXmlFeatures featureManager;
    static DocumentBuilderFactory dbf;
    static DocumentBuilder db;
    static Document d;
    
    protected XPathExpressionImpl() {
        this(null, null, null, null, false, new JdkXmlFeatures(false));
    }
    
    protected XPathExpressionImpl(XPath xpath, JAXPPrefixResolver prefixResolver, XPathFunctionResolver functionResolver, XPathVariableResolver variableResolver) {
        this(xpath, prefixResolver, functionResolver, variableResolver, false, new JdkXmlFeatures(false));
    }
    
    protected XPathExpressionImpl(XPath xpath, JAXPPrefixResolver prefixResolver, XPathFunctionResolver functionResolver, XPathVariableResolver variableResolver, boolean featureSecureProcessing, JdkXmlFeatures featureManager) {
        this.featureSecureProcessing = false;
        this.xpath = xpath;
        this.prefixResolver = prefixResolver;
        this.functionResolver = functionResolver;
        this.variableResolver = variableResolver;
        this.featureSecureProcessing = featureSecureProcessing;
        this.featureManager = featureManager;
        this.overrideDefaultParser = featureManager.getFeature(JdkXmlFeatures.XmlFeature.JDK_OVERRIDE_PARSER);
    }
    
    public void setXPath(XPath xpath) {
        this.xpath = xpath;
    }
    
    public Object eval(Object item, QName returnType) throws TransformerException {
        XObject resultObject = this.eval(item);
        return this.getResultAsType(resultObject, returnType);
    }
    
    private XObject eval(Object contextItem) throws TransformerException {
        XPathContext xpathSupport = null;
        if (this.functionResolver != null) {
            JAXPExtensionsProvider jep = new JAXPExtensionsProvider(this.functionResolver, this.featureSecureProcessing, this.featureManager);
            xpathSupport = new XPathContext(jep);
        }
        else {
            xpathSupport = new XPathContext();
        }
        xpathSupport.setVarStack(new JAXPVariableStack(this.variableResolver));
        XObject xobj = null;
        Node contextNode = (Node)contextItem;
        if (contextNode == null) {
            xobj = this.xpath.execute(xpathSupport, -1, this.prefixResolver);
            return xobj;
        }
        xobj = this.xpath.execute(xpathSupport, contextNode, this.prefixResolver);
        return xobj;
    }
    
    @Override
    public Object evaluate(Object item, QName returnType) throws XPathExpressionException {
        if (returnType == null) {
            String fmsg = XPATHMessages.createXPATHMessage("ER_ARG_CANNOT_BE_NULL", new Object[] { "returnType" });
            throw new NullPointerException(fmsg);
        }
        if (!this.isSupported(returnType)) {
            String fmsg = XPATHMessages.createXPATHMessage("ER_UNSUPPORTED_RETURN_TYPE", new Object[] { returnType.toString() });
            throw new IllegalArgumentException(fmsg);
        }
        try {
            return this.eval(item, returnType);
        }
        catch (NullPointerException npe) {
            throw new XPathExpressionException(npe);
        }
        catch (TransformerException te) {
            Throwable nestedException = te.getException();
            if ((nestedException instanceof XPathFunctionException)) {
                throw (XPathFunctionException)nestedException;
            }
            throw new XPathExpressionException(te);
        }
    }
    
    @Override
    public String evaluate(Object item) throws XPathExpressionException {
        return (String)this.evaluate(item, XPathConstants.STRING);
    }
    
    @Override
    public Object evaluate(InputSource source, QName returnType) throws XPathExpressionException {
        if (source == null || returnType == null) {
            String fmsg = XPATHMessages.createXPATHMessage("ER_SOURCE_RETURN_TYPE_CANNOT_BE_NULL", (Object[])null);
            throw new NullPointerException(fmsg);
        }
        if (!this.isSupported(returnType)) {
            String fmsg = XPATHMessages.createXPATHMessage("ER_UNSUPPORTED_RETURN_TYPE", new Object[] { returnType.toString() });
            throw new IllegalArgumentException(fmsg);
        }
        try {
            if (XPathExpressionImpl.dbf == null) {
                XPathExpressionImpl.dbf = JdkXmlUtils.getDOMFactory(this.overrideDefaultParser);
            }
            XPathExpressionImpl.db = XPathExpressionImpl.dbf.newDocumentBuilder();
            Document document = XPathExpressionImpl.db.parse(source);
            return this.eval(document, returnType);
        }
        catch (Exception e) {
            throw new XPathExpressionException(e);
        }
    }
    
    @Override
    public String evaluate(InputSource source) throws XPathExpressionException {
        return (String)this.evaluate(source, XPathConstants.STRING);
    }
    
    private boolean isSupported(QName returnType) {
        return returnType.equals(XPathConstants.STRING) || returnType.equals(XPathConstants.NUMBER) || returnType.equals(XPathConstants.BOOLEAN) || returnType.equals(XPathConstants.NODE) || returnType.equals(XPathConstants.NODESET);
    }
    
    private Object getResultAsType(XObject resultObject, QName returnType) throws TransformerException {
        if (returnType.equals(XPathConstants.STRING)) {
            return resultObject.str();
        }
        if (returnType.equals(XPathConstants.NUMBER)) {
            return new Double(resultObject.num());
        }
        if (returnType.equals(XPathConstants.BOOLEAN)) {
            return new Boolean(resultObject.bool());
        }
        if (returnType.equals(XPathConstants.NODESET)) {
            return resultObject.nodelist();
        }
        if (returnType.equals(XPathConstants.NODE)) {
            NodeIterator ni = resultObject.nodeset();
            return ni.nextNode();
        }
        String fmsg = XPATHMessages.createXPATHMessage("ER_UNSUPPORTED_RETURN_TYPE", new Object[] { returnType.toString() });
        throw new IllegalArgumentException(fmsg);
    }
    
    static {
        XPathExpressionImpl.dbf = null;
        XPathExpressionImpl.db = null;
        XPathExpressionImpl.d = null;
    }
}
